from torch.utils.data import Dataset
from pathlib import Path 
import numpy as np 
from PIL import Image
import torchvision.transforms as transforms

class DegradDataset(Dataset):
    """
     Degradation Resolution Dataset
     Require preprocessed original and degraded images (.png/.jpeg) files
     Return pairs of (original image, ldegraded image)
    """
    def __init__(self, 
                 img_dir, 
                 degrad, 
                 input_size=256, 
                 input_channel=3, 
                 suffix=".png",
                 length=None):
        self.img_dir = Path(img_dir)
        self.degrad = degrad
        self.suffix = suffix
        self.input_channel = input_channel
        orig_paths = list((Path(img_dir)/"original").glob(f"*{suffix}"))
        self.idx_lists = sorted(int(path.name.split('_')[-1].replace(suffix, "")) for path in orig_paths)
        if length is not None:
            self.idx_lists = self.idx_lists[:length]
        self.gt_transform = transforms.ToTensor()
        self.input_transform = transforms.Compose([
                        transforms.Resize(input_size),
                        transforms.ToTensor(),
                    ])
    def __len__(self):
        return len(self.idx_lists)
    def __getitem__(self, idx):
        orig_img = self.gt_transform(Image.open(self.img_dir/"original"/f"orig_{self.idx_lists[idx]}{self.suffix}"))
        degrad_img = self.input_transform(Image.open(self.img_dir/self.degrad/f"{self.degrad}_{self.idx_lists[idx]}{self.suffix}"))
        if self.input_channel < degrad_img.shape[1]:
            degrad_img = degrad_img[:self.input_channel]
        return orig_img, degrad_img