from torch.utils.data import Dataset
from pathlib import Path 
import numpy as np 
import torchvision.transforms as transforms
from PIL import Image

# ----- Ref: https://github.com/bahjat-kawar/ddrm ----- #
# ----- ddrm/datasets/imagenet_subset.py          ----- #
class CenterCropLongEdge(object):
    """Crops the given PIL Image on the long edge.
    Args:
        size (sequence or int): Desired output size of the crop. If size is an
            int instead of sequence like (h, w), a square crop (size, size) is
            made.
    """

    def __call__(self, img):
        """
        Args:
            img (PIL Image): Image to be cropped.
        Returns:
            PIL Image: Cropped image.
        """
        return transforms.functional.center_crop(img, min(img.size))

    def __repr__(self):
        return self.__class__.__name__

class ImageDataset(Dataset):
    """
     Image Resolution Dataset
     Read an directory that contains original images (.png/.jpeg)
     Crop images at long side and resize them to the target resolution
    """
    def __init__(self, img_dir, size, suffix=".png"):
        self.img_dir = Path(img_dir)
        self.img_paths = sorted(list(Path(img_dir).glob(f"*{suffix}")))
        self.transform = transforms.Compose([
                    CenterCropLongEdge(),
                    transforms.Resize(size),
                    transforms.ToTensor(),
                ])
    def __len__(self):
        return len(self.img_paths)
    def __getitem__(self, idx):
        img = Image.open(self.img_paths[idx]).convert("RGB")
        img = self.transform(img)
        return img