from diffusers import LDMSuperResolutionPipeline # version 0.11.1
from .CustomLDMSuperResolutionPipeline import CustomLDMSuperResolutionPipeline
class LDM_pipeline:
    def __init__(self, 
                ldm_name="CompVis/ldm-super-resolution-4x-openimages",
                eta=1,
                num_inference_steps=100,
                return_intermediate_results=False,
                device="cuda"):
        if return_intermediate_results:
            self.pipeline = CustomLDMSuperResolutionPipeline.from_pretrained(ldm_name).to(device)
        else:
            self.pipeline = LDMSuperResolutionPipeline.from_pretrained(ldm_name).to(device)
        self.return_intermediate_results = return_intermediate_results
        self.num_inference_steps = num_inference_steps
        self.eta = eta 
        self.device = device
    def __call__(self, batch):
        batch = batch.to(self.device)
        output = self.pipeline(batch, num_inference_steps=self.num_inference_steps, eta=self.eta, output_type="ndarray").images
        if self.return_intermediate_results:
            print("len output:", len(output))
            return output[-1], output
        return output, None

